//
//
//
//
//
//
unit Unit_Base;

interface

Uses
 Types,Messages,Windows,Classes,SysUtils,ShellAPI,Forms,Math,
 VCL_USB_HID,Unit_IntelHex;

Const
 cMax_PktCnt = 14;
Type
 TIntelHex  = Record
               Hex  : TStringList;
               Bin  : TMemoryStream;
               Addr : Int64;
               //
               Dat  : Packed Array[0..1024*1024*2-1] of Byte; // Max 3MB
              End;
 TUSB_Buf  = Packed Array[0..64-1] of Byte;
 TUSB_Pkt  = Packed Record
              ID    : Byte;
              Buf   : TUSB_Buf;
             End;
 TPkt      = Packed Record
              Cmd       : Byte;  // Unlock , Erase , Write
              Cnt       : Byte;  // Dat Count
              Reserved1 : Byte;
              Reserved2 : Byte;
              Addr      : DWord; // Addr
              Dat       : Array[0..cMax_PktCnt-1] of DWord; // DWord x 14 = 56 Bytes
             End;
 TUSB      = Record
              Dev       : THIDDevice;
              //
              SndBuf    : TUSB_Buf;  // Send    Buffer
              RcvBuf    : TUSB_Buf;  // Receive Buffer
              //
              Rcved     : Boolean;
              RcvStr    : String;
             End;

//
Procedure DragDrop_Init ( Handle : THandle );
Procedure DragDrop_Process ( Var msg : TWMDropFiles; Files : TStrings );
//
Procedure IntelHex_Create ( Var Bin : TIntelHex );
Procedure IntelHex_Free   ( Var Bin : TIntelHex );
Function  IntelHex_Load   ( Var Bin : TIntelHex; FileName : String ) : Boolean;
//
Procedure USB_Init        ( Var USB : TUSB);
Procedure USB_Connect     ( Var USB : TUSB; Dev : THidDevice );
Procedure USB_Disconnect  ( Var USB : TUSB );
Function  USB_Read        ( Var USB : TUSB; Data : Pointer; Size : Word ) : Boolean;
Function  USB_WriteStr    ( Var USB : TUSB; Str : String  ; WaitCh : String = '' ) : Boolean;
Function  USB_WriteBuf    ( Var USB : TUSB;                 WaitCh : String = '' ) : Boolean;

Const
 _cAddr_Base   : DWord = $08000000;  // Base   Address
 _cAddr_Loader : DWord = $081E0000;  // $080C0000;  // BootLoader Address

 _cDevVID      = $0483;  // XRef. _HAL_STM32H7xx_USB.mpas #1119  USBD_VID    = 0x0483;
 _cDevHID      = $5750;  //       _HAL_STM32H7xx_USB.mpas #1120  USBD_PID_FS = 0x5750;


Var
 _gHex : TIntelHex;
 _gUSB : TUSB;

implementation

//-----------------------------------------------------------------------------
// DragDrop
//-----------------------------------------------------------------------------

Procedure DragDrop_Init ( Handle : THandle );
 Const
  WM_COPYGLOBALDATA = $0049;
  MSGFLT_ADD        = 1;
  MSGFLT_ALLOW      = 1;

 Var
  ChangeWindowMessageFilter : function(message : UINT; dwFlag : DWORD) : BOOL; stdcall;

 begin
  DragAcceptFiles( Handle, True );
  //
  @ChangeWindowMessageFilter   := GetProcAddress(GetModuleHandle(user32),'ChangeWindowMessageFilter');
  //
  If  Assigned(ChangeWindowMessageFilter  ) then
   begin
    ChangeWindowMessageFilter(WM_DROPFILES,MSGFLT_ADD);
    ChangeWindowMessageFilter(WM_COPYDATA,MSGFLT_ADD);
    ChangeWindowMessageFilter(WM_COPYGLOBALDATA,MSGFLT_ADD);
   end;
 end;

//
// procedure WMDROPFILES(var msg : TWMDropFiles) ; message WM_DROPFILES;
//
Procedure DragDrop_Process ( Var msg : TWMDropFiles; Files : TStrings );
 Const
  cMaxFileName = 255;
 Var
  i        : Integer;
  CntFile  : Integer;
  fileName : array [0..cMaxFileName] of char;
 begin
  //
  Files.Clear;
  //
  CntFile := DragQueryFile(msg.Drop, $FFFFFFFF, fileName, cMaxFileName );
   For i := 0 to CntFile-1 do
    begin
     DragQueryFile(msg.Drop, i, fileName, cMaxFileName );
     Files.Add(fileName);
    end;
  DragFinish(msg.Drop) ;
  //
 end;

//-----------------------------------------------------------------------------
// IntelHex
//-----------------------------------------------------------------------------

Procedure IntelHex_Create ( Var Bin : TIntelHex );
 begin
  Bin.Hex  := TStringList.Create;
  Bin.Bin  := TMemoryStream.Create;
  Bin.Addr := 0;
 end;

Procedure IntelHex_Free   ( Var Bin : TIntelHex );
 begin
  Bin.Hex.Free;
  Bin.Bin.Free;
  Bin.Addr := 0;
 end;

Function  IntelHex_Load   ( Var Bin : TIntelHex; FileName : String ) : Boolean;
 begin
  Result := False;
  If Not( FileExists(FileName) ) then Exit;
  //
  Bin.Hex.LoadFromFile ( FileName );
  Hex2Bin (Bin.Hex,Bin.Bin,Bin.Addr);
  //
  Result := Bin.Bin.Size > 0;
 end;

//-----------------------------------------------------------------------------
//
// USB Base Functions
//
//-----------------------------------------------------------------------------

Procedure USB_Init ( Var USB : TUSB );
 begin
  USB.Dev   := nil;
  USB.Rcved := False;
 end;

Procedure USB_Connect     ( Var USB : TUSB; Dev : THidDevice );
 begin
  USB.Dev := Dev;
  USB.Dev.CheckOut;
 end;

Procedure USB_Disconnect  ( Var USB : TUSB );
 begin
  //
  //If USB.Dev = nil then Exit;
  //
  //USB.Dev.CloseFile;
  //USB.Dev.CheckOut;
  //USB.Dev             := nil;
 end;

Function  USB_Read        ( Var USB : TUSB; Data : Pointer; Size : Word ) : Boolean;
 Var
  i : Integer;
 begin
  FillChar(USB.RcvBuf,SizeOf(USB.RcvBuf),#0);
  Move    (Data^,USB.RcvBuf,Min(SizeOf(USB.RcvBuf),Size));
  //
  USB.RcvStr := '';
  For i := 0 to SizeOf(USB.RcvBuf)-1 do
   Case USB.RcvBuf[i] = 0 of
    True  : Break;
    False : USB.RcvStr := USB.RcvStr + Char(USB.RcvBuf[i]);
   End;
  //
  USB.Rcved := True;
  Result    := Size > 0;
 end;

Function  USB_WriteStr    ( Var USB : TUSB; Str : String  ; WaitCh : String = '' ) : Boolean;
 Var
  Pkt     : TUSB_Pkt;
  i       : Integer;
  ToWrite : Cardinal;
  Written : Cardinal;
 begin
  //
  Result := False;
  If USB.Dev = nil then Exit;
  //
  Pkt.ID  := 0;
  FillChar(Pkt.Buf,SizeOf(Pkt.Buf),#0);
  For i := 1 to Length(Str) do
   Case Ord(Str[i]) = 0 of
    True  : Break;
    False : Pkt.Buf[i-1] := Ord(Str[i]);
   End;
  //
  ToWrite := USB.Dev.Caps.OutputReportByteLength;
  Result  := USB.Dev.WriteFile(Pkt, ToWrite, Written);

  // Wait Case
  If Result and (WaitCh <> '') then
   begin
    USB.Rcved := False;
    Repeat
     Application.ProcessMessages;
     If ( USB.Rcved ) and
        ( USB.RcvBuf[0] = Ord(WaitCh[1]) ) then
      begin
       Result := True;
       Exit;
      end;
    Until ( False );
   end;
 end;

Function  USB_WriteBuf  ( Var USB : TUSB;                 WaitCh : String = '' ) : Boolean;
 Var
  Pkt     : TUSB_Pkt;
  i       : Integer;
  ToWrite : Cardinal;
  Written : Cardinal;
 begin
  //
  Result := False;
  If (USB.Dev = nil) then Exit;
  //
  Pkt.ID  := 0;
  Pkt.Buf := USB.SndBuf;
  //
  ToWrite := USB.Dev.Caps.OutputReportByteLength;
  Result  := USB.Dev.WriteFile(Pkt, ToWrite, Written);

  // Wait Case
  If Result and (WaitCh <> '') then
   begin
    USB.Rcved := False;
    Repeat
     Application.ProcessMessages;
     If ( USB.Rcved ) and
        ( USB.RcvBuf[0] = Ord(WaitCh[1]) ) then
      begin
       Result := True;
       Exit;
      end;
    Until ( False );
   end;
 end;


Initialization
 IntelHex_Create ( _gHex );
Finalization
 IntelHex_Free   ( _gHex );

end.
