//
//
//
//
//
//
unit Unit_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls,Gauges,ExtCtrls,Math,
  //
  Unit_Base, VCL_USB_HID ;

type
  //
  TForm_Main = class(TForm)
    Memo_Dbg: TMemo;
    SB_Bottom: TStatusBar;
    Gauge: TGauge;
    USB: TUSBHID;
    Pnl_Top: TPanel;
    Lbl_Flash: TLabel;
    Ed_Flash: TEdit;
    Pnl_Active: TPanel;
    Btn_Jump_Loader: TButton;
    Btn_Download: TButton;
    Btn_Reload: TButton;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure USBArrival(HidDev: THidDevice);
    procedure USBDeviceData(HidDev: THidDevice; ReportID: Byte; const Data: Pointer; Size: Word);
    procedure USBDeviceUnplug(HidDev: THidDevice);
    procedure Btn_Jump_LoaderClick(Sender: TObject);
    procedure Btn_DownloadClick(Sender: TObject);
    procedure Btn_ReloadClick(Sender: TObject);
    procedure USBDeviceChange(Sender: TObject);
    procedure USBDeviceDataError(HidDev: THidDevice; Error: Cardinal);
    function USBEnumerate(HidDev: THidDevice; const Idx: Integer): Boolean;
    procedure USBRemoval(HidDev: THidDevice);
  private
    procedure WMDROPFILES(var msg : TWMDropFiles) ; message WM_DROPFILES;
  public
    _gFirstTime : Boolean;
    _gFileName  : String;
    //
    Procedure Dbg( Const Msg : String );
    Function  LoadHexBin ( FileName : String ) : Boolean;
  end;

var
  Form_Main: TForm_Main;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
//
// Utility
//
//------------------------------------------------------------------------------

Procedure TForm_Main.Dbg( Const Msg : string );
 begin
  If Memo_Dbg.Lines.Count > 5000 then Memo_Dbg.Lines.Clear;
  //
  Memo_Dbg.Lines.Add( Msg );
 end;

Function  TForm_Main.LoadHexBin ( FileName : String ) : Boolean;
 begin
  //
  Result := False;
  If Not(FileExists(FileName)) then Exit;

  //
  IntelHex_Load ( _gHex, FileName );
  _gFileName := FileName;
  //
  Ed_Flash.Text := ExtractFileName ( FileName );
  Memo_Dbg.Lines.Clear;
  Dbg( 'Loaded : '          + ExtractFileName( FileName ) );
  Dbg( 'Start Address : 0x' + IntToHex(_gHex.Addr,8) );
  Dbg( 'Size : 0x'          + IntToHex ( _gHex.Bin.Size , 8 ) + ' (' +
                              IntToStr ( _gHex.Bin.Size div 1024 ) + 'KB)');
  //
  Result := True;
 end;

procedure TForm_Main.WMDropFiles (Var msg : TWMDropFiles);
 Var
  i        : Integer;
  Strs     : TStrings;
  FileName : String;
 begin
  //
  Strs := TStringList.Create;
  DragDrop_Process(msg,Strs);
  //
  For i := 0 to Strs.Count-1 do
   begin
    FileName := Strs[i];
    If Pos('.HEX', UpperCase(FileName) ) > 0 then
     begin
      LoadHexBin ( FileName );
      Break;
     end;
    //
    Application.ProcessMessages;
   end;
  Strs.Free;
 end;


//------------------------------------------------------------------------------
//
// Form Event
//
//------------------------------------------------------------------------------

procedure TForm_Main.FormCreate(Sender: TObject);
begin
 // IntelHex_Create (_gIntelHex );
 DragDrop_Init   (Self.Handle);
 //
 USB_Init(_gUSB);
 //
 _gFirstTime := True;
end;

procedure TForm_Main.FormDestroy(Sender: TObject);
begin
 USB.OnDeviceUnplug := nil;
end;

procedure TForm_Main.FormActivate(Sender: TObject);
 Var
  FileName : String;
begin
 If Not( _gFirstTime ) then Exit;
 _gFirstTime := False;
 //
 FileName := '.\..\..\Lpix.hex';
 LoadHexBin(FileName);
end;


//-----------------------------------------------------------------------------
//
// USB Event
//
//-----------------------------------------------------------------------------

procedure TForm_Main.USBArrival(HidDev: THidDevice);
begin
  Dbg('Event::OnArrival -> '+ HidDev.ProductName );
  //
  If (HidDev.Attributes.VendorID  = _cDevVID  ) and
     (HidDev.Attributes.ProductID = _cDevHID  ) then
   //  (HidDev.ProductName          = _cDevBoot ) then
  begin
   //
   USB_Connect(_gUSB,HidDev);

   // UI
   // Dbg( '>> ' + HidDev.ProductName  );
   Pnl_Active.Caption := HidDev.ProductName;
   Pnl_Active.Color   := clLime;
  end;
end;

procedure TForm_Main.USBDeviceChange(Sender: TObject);
begin
  Dbg('Event::OnDeviceChange');
end;

procedure TForm_Main.USBDeviceData(HidDev: THidDevice; ReportID: Byte;
   const Data: Pointer; Size: Word);
begin
 Dbg('Event::OnDeviceData -> ' + HidDev.ProductName );
 //
 If USB_Read( _gUSB, Data, Size ) then
  Case _gUSB.RcvStr = 'w' of
   True  : ;
   False : Dbg(_gUSB.RcvStr);
  End;
end;

procedure TForm_Main.USBDeviceDataError(HidDev: THidDevice; Error: Cardinal);
begin
 Dbg('Event::OnDeviceDataError -> ' + HidDev.ProductName );
end;

procedure TForm_Main.USBDeviceUnplug(HidDev: THidDevice);
begin
  Dbg('Event::OnDeviceUnplug -> ' + HidDev.ProductName );
  //
  If (HidDev.Attributes.VendorID  = _cDevVID  ) and
     (HidDev.Attributes.ProductID = _cDevHID  ) then
  begin
   USB_Disconnect(_gUSB);
   // UI
   // Dbg( '<< ' + HidDev.ProductName  );
   Pnl_Active.Caption := '';
   Pnl_Active.Color   := clRed;
  end;
end;

function TForm_Main.USBEnumerate(HidDev: THidDevice; const Idx: Integer): Boolean;
begin
  Dbg('Event::OnUSBEnumerate' + HidDev.ProductName + ' ' + IntToStr(Idx) );
end;

procedure TForm_Main.USBRemoval(HidDev: THidDevice);
begin
  Dbg('Event::OnUSBRemoval   -> ' + HidDev.ProductName );
end;

//-----------------------------------------------------------------------------
//
// Button Event
//
//-----------------------------------------------------------------------------

procedure TForm_Main.Btn_ReloadClick(Sender: TObject);
begin
 LoadHexBin( _gFileName );
end;

procedure TForm_Main.Btn_DownloadClick(Sender: TObject);
 Var
  Pkt      : TPkt;
  Pc       : DWord;
  SpPC     : Array[0..1] of DWord; // SP,PC
  Send     : Integer;
  i        : Integer;
begin

 // Step #1. Move
 Move(_gHex.Bin.Memory^,_gHex.Dat[0],_gHex.Bin.Size);
 Move(_gHex.Dat[0],SpPc,SizeOf(SpPc));     // Save SP & PC

 // Set Start to BootLoader
 PC := _cAddr_Loader + 1;                  // New PC
 Move(PC,_gHex.Dat[4],SizeOf(_cAddr_Loader));

 // Step #2. Erase ------------------------------------------------------------
 Dbg('Flash Erased');
 USB_WriteStr(_gUSB,'E','e');

 // Step #3. Writing App ------------------------------------------------------
 Dbg('Flash Writing');
 Send := 0;
 Repeat
  //
  Pkt.Cmd       := Ord('W');
  Pkt.Cnt       := 8; //(SizeOf(Pkt.Dat) div 4); // DWord Aligned
  Pkt.Reserved1 := 0;
  Pkt.Reserved2 := 0;
  Pkt.Addr      := $08000000 + Send;
  Move(_gHex.Dat[Send], Pkt.Dat, 8*4 ); // SizeOf(Pkt.Dat) );
  Move(Pkt,_gUSB.SndBuf,SizeOf(_gUSB.SndBuf));
  USB_WriteBuf(_gUSB,'w');
  //
  Inc ( Send , 8*4 ); // SizeOf(Pkt.Dat) );

  //Dbg ( 'Write ' + IntToHex(Send,8) );
  Gauge.MaxValue := _gHex.Bin.Size;
  Gauge.Progress := Send;
  //
 Until (Send >= _gHex.Bin.Size );

 // Step #4. Writing App Info ( Save SP,PC before Loader Addr )
 Pkt.Cmd       := Ord('W');
 Pkt.Cnt       := 8; // 2 DWord (SP,PC)
 Pkt.Reserved1 := 0;
 Pkt.Reserved2 := 0;
 Pkt.Addr      := _cAddr_Loader - 32; // 0x081DFFE0

 Pkt.Dat[0]    := $FFFFFFFF;          //
 Pkt.Dat[1]    := $FFFFFFFF;          //
 Pkt.Dat[2]    := $FFFFFFFF;          //
 Pkt.Dat[3]    := $FFFFFFFF;          //
 Pkt.Dat[4]    := $FFFFFFFF;          //
 Pkt.Dat[5]    := $FFFFFFFF;          //
 Pkt.Dat[6]    := SpPc[0];            // SP
 Pkt.Dat[7]    := SpPc[1];            // PC
 Move(Pkt,_gUSB.SndBuf,SizeOf(_gUSB.SndBuf));
 USB_WriteBuf(_gUSB,'w');

 // Step #4. Reset ------------------------------------------------------------
 For i := 0 to 100 do
  begin
   Sleep(1);
   Application.ProcessMessages;
  end;

 Dbg('Jump to App');
 USB_WriteStr(_gUSB,'J');

 //
 Gauge.Progress := 0;
 Dbg('Done');
end;

procedure TForm_Main.Btn_Jump_LoaderClick(Sender: TObject);
begin
 USB_WriteStr(_gUSB,'J');
end;


end.
